
/*********************************************************************\

                      CALL LOG SUMMARY PROGRAM Version 1.01
                          For use with PROCOMM PLUS

                                Mike Nice
                                  5/88

                          (Compuserve 73565,565)

   This program was compiled using Turbo C and the COMPACT memory
   model to allow processing of log files up to the size of
   available RAM.

DESCRIPTION:

	SUMMLOG reads a call log file created by PROCOMM PLUS
and summarizes the information by month, year or combined total.
A report is produced for each number called as follows:

  o Total number of times called.
  o Total connect time for that number.
  o Number of entries with no time entered in the 'OFFLINE' field of the
    PROCOMM PLUS log file are listed under 'NOT LOGGED' in the output file.
    This may occur if the computer was rebooted or turned off while online,
    or if the ALT-X option was used to exit PROCOMM PLUS while still online.

\**************************************************************************/


#include <stdio.h>
#include <stdlib.h>
#include <alloc.h>
#include <string.h>
#include <conio.h>
#include <ctype.h>

#define FALSE 0
#define TRUE -1
#define CR 13
#define BS 8
#define CNTLC 3
#define ESC 27
#define MaxLines 8000		/* 80 char/line * 8000 lines = 640 k */



   /* Storage structure for each log entry */

typedef struct _LogEntry {
  char datePos;        /* Position of date called field */
  char logEntry[];     /* ASCII text of entry */
} LogEntry;

  /* Declare function prototypes */

int compare( LogEntry **s1, LogEntry **s2);
LogEntry **ReadFile(FILE *inFile, int *numEntries);
long loggedTime(char *entry, int *numNotLogged, int datePosition);

typedef enum {grandTotal, yearly, monthly} SummaryTypes;

SummaryTypes logSummaryType;   /* Global summary type variable */

char firstCallDate[9], lastCallDate[9];

char *monthName[] = { "","January","February","March","April",
                      "May","June","July","August","September",
                      "October","November","December"} ;


main(int argc, char *argv[]) {

  LogEntry **logPtr;   /* Dynamic array of pointers to log entries */
  int numElements;	      /* Number of log entries */
  FILE *inFile;
  int argUsed;
  char argVal[80];
  char fileName[80];

    /* Force unbuffered output on stdout device.  For some reason
       TC v 1.0 buffers std output in DOS compatibility box of OS/2 */
  setbuf(stdout, NULL);

  printf("\nProComm Plus call log summary V 1.01  5/88\n");
  printf("by Mike Nice\n\n");


  /* If command line arguments are entered, they will be used
     instead of prompting for input */

  argUsed = FALSE;
  do {

    if ((argc > 1) && !argUsed) {					 /* use command line parameter */
      strcpy(fileName,argv[1]);
      printf("Log filename: %s",fileName);
      argUsed = TRUE;
    }
    else {                                    /* ask for filename */
      printf("Enter log filename <PCPLUS.FON>: ");
      readln(fileName,79);
      if (fileName[0] == 0) {
        strcpy(fileName,"PCPLUS.FON");
        printf("PCPLUS.FON");
      }
   }

    if ( (inFile = fopen(fileName,"r")) == NULL)
      printf (" --- file not found\n\n");
  } while (inFile == NULL);

  if (argc > 2) strcpy(argVal,argv[2]); else argVal[0]=0;
  GetSummaryType(&logSummaryType,argVal);

  if (argc > 3) strcpy(argVal,argv[3]); else argVal[0]=0;
  GetOutputFilename(fileName,argVal);

  printf("\nReading ...\n");

  logPtr = ReadFile(inFile, &numElements);

  if (logPtr == NULL)  {
    printf("Not enough memory.\n");
    printf("If you are running from PROCOMM's DOS gateway (ALT-F4),\n");
    printf("try again after exiting PROCOMM using ALT-X.\n");
    exit(1);
  }

  printf("Sorting entries ...\n");

  qsort( logPtr,numElements,4, compare );

  printf("Writing results...\n");

  TabulateResults(fileName, logPtr, numElements, TRUE);  /* Yearly summary */

  exit(0);
}


GetSummaryType(SummaryTypes *summaryType, char *argVal) {

int selected;
char ch, sel;

  sel = toupper(argVal[0]);
  printf("\n\nChoose summary type -\n");
  printf("  <C>ombined summary\n");
  printf("  by <Y>ear\n");
  printf("  by <M>onth\n\n");
  printf("Enter C, Y, or M  <C>: ");

  selected = FALSE;
  do {
    if (!sel) ch=toupper(getch()); else { ch=sel; sel=0; }
    switch (ch) {
      case CR  : ch='C';
      case 'C' : *summaryType = grandTotal; selected = TRUE; break;

      case 'Y' : *summaryType = yearly;  selected = TRUE; break;

      case 'M' : *summaryType = monthly; selected = TRUE; break;

      case CNTLC :
      case ESC : exit(1);
    }
  } while (!selected);
  printf("%c\n",ch);
}

GetOutputFilename(char *fileName, char *argVal) {

  printf("Enter output filename <LOG.ALL>: ");

  if (argVal[0]) {        /* If command line entry */
    printf("%s",argVal);
    strcpy(fileName,argVal);
  }
  else readln(fileName,79);

  if (fileName[0]==0) {      /* If <RETURN> was pressed */
    strcpy(fileName,"LOG.ALL");
    printf("LOG.ALL");
  }
  printf("\n");
}



LogEntry **ReadFile(FILE *inFile, int *numEntries){

  int datePosition;
  int lineLen, numElements;
  LogEntry *curPtr;        /* Current position in Text buffer */
  LogEntry **logPtr;       /* Dynamic array of pointers to log entries */
  char huge *hugePtr;
  int firstEntry, isValidEntry;

  firstCallDate[0]=0;
  lastCallDate[0]=0;

  firstEntry = TRUE;
  fseek(inFile, 0L, SEEK_END);           /* Determine file size */
  curPtr = farmalloc(ftell(inFile)+2L);  /* File length + null terminator */
  if (curPtr == NULL) {
    fclose(inFile);		      /* Insufficient memory */
    return(NULL);
  }
  fseek(inFile,0L, SEEK_SET);          /* Reset to beginning */

  logPtr = calloc( MaxLines , sizeof(LogEntry *) );
  if (logPtr == NULL) {
    fclose(inFile);		      /* Insufficient memory */
    farfree(curPtr);                  /* Free previous block */
    return(NULL);
  }


  /* Large buffer for input */
  setvbuf(inFile, NULL, _IOFBF, 16384);

  /* Read file into memory, filling in array of pointers to each line */

  hugePtr = (char huge *) curPtr;
  numElements = 0;
  while (((fgets(&curPtr->logEntry, 255, inFile)) != NULL)
           && (numElements < MaxLines)) {
    CheckForEntry(curPtr, &lineLen, &isValidEntry);
    if (isValidEntry) {
      logPtr[numElements] = curPtr;
      if (firstEntry) {
        datePosition = curPtr->datePos;
        strncpy(firstCallDate,&curPtr->logEntry[datePosition],8);
        firstCallDate[8]=0;
        firstEntry = FALSE;
      }

       /* Point to next buffer.  Overwrite terminating null with datePos */
      hugePtr +=  lineLen +1;      /* Normalize pointer */
      curPtr = (LogEntry *)hugePtr ;
      numElements++;
    }
  }

  fclose(inFile);

  if (numElements>0)  {
    datePosition = logPtr[numElements-1]->datePos;
    strncpy(lastCallDate, &logPtr[numElements-1]->logEntry[datePosition], 8);
    lastCallDate[8]=0;
  }

  *numEntries = numElements;
  return(logPtr);
}


/* Examine each line as read from file.  To be valid it must
   be at least 54 characters long, have a valid date-time
   string in the form "mm/dd/yy hh:mm:ss".
                       ^ ^
                       | |- dateSlashPos
                       |----datePos
   The starting position of the date string is recorded at
   'datePos' */

CheckForEntry(LogEntry *entryPtr, int *lineLen, int *isValidEntry) {

char *dateSlashPos;

  *lineLen = strlen(entryPtr->logEntry);
  *isValidEntry = FALSE;

  if ((*lineLen < 54) ||
     (strstr(entryPtr->logEntry,"ONLINE  OFFLINE  ELAPSED")!=NULL) ) return;

  dateSlashPos = strchr(&entryPtr->logEntry[46],'/');
  if ( (dateSlashPos == NULL) ||
       (dateSlashPos[3] != '/') ||
       (dateSlashPos[9] != ':') ) return;
  entryPtr->datePos = dateSlashPos - entryPtr->logEntry - 2;
  *isValidEntry = TRUE;
}




/* Compare function called from qsort */

int compare(LogEntry **s1, LogEntry **s2)
{
int mismatch,
    yearPos1, monthPos1,
    yearPos2, monthPos2;



  switch (logSummaryType) {
  case grandTotal:                 /* Sort priority = name */
    return( strnicmp(s1[0]->logEntry, s2[0]->logEntry,50) );

  case yearly:                     /* Sort = year, name */
    yearPos1 = s1[0]->datePos + 6;
    yearPos2 = s2[0]->datePos + 6;
    mismatch = strncmp(&s1[0]->logEntry[yearPos1],
                       &s2[0]->logEntry[yearPos2], 2);
    if (!mismatch)
      return (strnicmp (s1[0]->logEntry, s2[0]->logEntry,50) );     /* Name */
    return (mismatch);

  case monthly:                    /* Sort = year, month, name */
    monthPos1 = s1[0]->datePos;
    monthPos2 = s2[0]->datePos;
    yearPos1 = monthPos1 + 6;
    yearPos2 = monthPos2 + 6;
    mismatch = strncmp(&s1[0]->logEntry[yearPos1],
                      &s2[0]->logEntry[yearPos2], 2);
    if (!mismatch) {
      mismatch = strncmp(&s1[0]->logEntry[monthPos1],
                          &s2[0]->logEntry[monthPos2], 2);
      if (!mismatch)
        return (strnicmp (s1[0]->logEntry, s2[0]->logEntry,50) );  /* Name */
    }
  }
  return(mismatch);
}



/* Calculate total logged time, # logons, write results to file */

TabulateResults(char *fileName, LogEntry **logPtr, int numElements) {

FILE *outFile;
char refEntry[255];
int i;
int refDatePos, datePosition;
int numCalls, numNotLogged, newLogType, lineLength;
unsigned long seconds;
int firstEntry;

  if ((outFile=fopen(fileName,"w")) == NULL) {
    printf("Unable to create file %s\n",fileName);
    exit(1);
  }
  printf("Writing %s\n",fileName);

  firstEntry = TRUE;
  for (i=0; i<numElements; i++){
    lineLength = strlenLF(logPtr[i]->logEntry);
    datePosition = logPtr[i]->datePos;
    if (firstEntry) {
      firstEntry = FALSE;
      strncpy(refEntry, logPtr[i]->logEntry, lineLength);
      refDatePos = datePosition;
      WriteHeader(outFile, refEntry, datePosition);
      numNotLogged = 0;
      seconds = loggedTime(logPtr[i]->logEntry, &numNotLogged, datePosition);
      numCalls = 1;
    }
    else {
      if ( NewEntry(refEntry, logPtr[i]->logEntry, &newLogType,
                    datePosition, refDatePos)) {

        /* Have new entry, output previous results */
        OutputResults(outFile,refEntry,seconds,numCalls,
                      numNotLogged,refDatePos);
        strncpy(refEntry, logPtr[i]->logEntry, lineLength);
        refDatePos = datePosition;

        if (newLogType) {
          fprintf(outFile,"\n\n");
          WriteHeader(outFile, refEntry, refDatePos);
        }

        numNotLogged = 0;
        seconds = loggedTime(logPtr[i]->logEntry,
                             &numNotLogged, datePosition);
        numCalls = 1;
      }
      else {
        /* Accumulate time for that entry */
        seconds += loggedTime(logPtr[i]->logEntry,
                              &numNotLogged, datePosition);
        numCalls++;
      }
    }
  }
  /* Write last record */
  if (numElements>0)
    OutputResults(outFile,refEntry,seconds,numCalls,numNotLogged,datePosition);
  fclose(outFile);
}


/* See if this entry is new for this log type:
   eg.  if monthly summary, is this a new month?
   Also, is this the same name and number?
 */
int NewEntry(char *refEntry, char *testEntry, int *newLogType,
             int datePosition, int refDatePos) {

int newLogEntry;

  *newLogType = FALSE;
  newLogEntry = strnicmp(refEntry,testEntry,datePosition-1);

  switch (logSummaryType) {
  case yearly:
     *newLogType = strncmp(&refEntry[refDatePos+6],
                           &testEntry[datePosition+6],2);
     break;
  case monthly:
     *newLogType = strncmp(&refEntry[refDatePos],&testEntry[datePosition],2);
     break;
  }
  return (*newLogType || newLogEntry);
}



/* Write appropriate output header to file */

WriteHeader(FILE *outFile, char *entry, int datePosition) {

int month, invalid;
char year[3];

  strncpy(year,&entry[datePosition + 6],2);
  year[2]=0;
  fprintf(outFile,"\nSummary of ProComm call log ");
  switch (logSummaryType) {
  case grandTotal:
     fprintf(outFile,"from %s to %s\n",firstCallDate,lastCallDate);
     break;
  case yearly:
     fprintf(outFile,"for 19%s\n",year);
     break;
  case monthly:
     month=convert(&entry[datePosition],&invalid);
     fprintf(outFile,"for %s, 19%s\n",monthName[month],year);
     break;
  }
  fprintf(outFile,"\nNAME                                 NUMBER");
  fprintf(outFile,"     CALLS      TOTAL    NOT LOGGED\n");
}


/*  Extract time on, time off from entry, calculate elapsed time.
    If unable to get times, increment numNotLogged.

    ONLINE  is at entry[datePosition + 9] in hh:mm:ss
    OFFLINE is at entry[datePosition + 18] in hh:mm:ss
 */
long loggedTime(char *entry, int *numNotLogged, int datePosition) {

long online, offline, elapsed;
int notLogged;

  notLogged = FALSE;

  /* Extract online time */
  online =  convert(&entry[datePosition + 9], &notLogged) * 3600L;
  online += convert(&entry[datePosition + 12], &notLogged) * 60L;
  online += convert(&entry[datePosition + 15], &notLogged);

  /* Extract offline time */
  offline =  convert(&entry[datePosition + 18], &notLogged) * 3600L;
  offline += convert(&entry[datePosition + 21], &notLogged) * 60L;
  offline += convert(&entry[datePosition + 24], &notLogged);

  if (offline < online) offline += 86400L;   /* Check for midnight wrap */

  elapsed = offline - online;

  if (notLogged) {
    elapsed =0L;
    (*numNotLogged)++;
  }

  return(elapsed);

}


/* Convert 2 next ascii digits in string to int, set invalid flag if
   not numeric */

int convert(char *number, int *invalid) {

char num[3];

  num[0] = number[0];
  num[1] = number[1];
  num[2] = 0;

  if ( !isdigit(num[0]) || !isdigit(num[1]) ) {
    *invalid = TRUE;
  }
  return (atoi(num));
}


/* Print one line of results to output file */

OutputResults(FILE *outFile, char *entry, long seconds,
              int numCalls, int numNotLogged, int datePosition) {

char line[80];
int hh;
int mm;
int ss;

  /* Convert total seconds to HH:MM:SS */

  hh = seconds / 3600L;
  seconds = seconds % 3600L;

  mm = seconds / 60L;
  ss = seconds % 60L;

  strncpy(line,entry,datePosition-1);

  /* Make all names 46 chars long */
  strncpy(&line[datePosition-1],"      ",6);
  line[46]=0;

  fprintf(outFile,"%s  %5d   %3d:%02d:%02d   %5d\n",
          line, numCalls, hh, mm, ss, numNotLogged);
}



/* Read line of input with echo, don't do CR or LF */

readln(char *st,int maxnum)
{
char ch;
int pos;

  pos=0;
  do {
    ch=getch();
    switch (ch) {
      case BS : if (pos>0) {
                 pos--;
                 printf("%c %c",ch,ch);
                 break;
               }
      case CR : st [pos]=0; break;
      case CNTLC  :
      case ESC    : exit(1);
      default : if ( (pos < maxnum) && isprint(ch) ) {
                     st[pos++]=ch; putchar(ch);
                }
    }
  } while (ch != CR);
}


/* Compute length of string terminated by NULL or lineFeed.
   This is used because terminating nulls were overwritten
   as the file was read.  The last line may not have a lineFeed,
   so NULLs are checked for also. */

int strlenLF(char *st) {

int length;
char *p;

  p=st;
  length=0;
  while (*p && (*p != 0x0a)) {
    p++;
    length++;
  }
  return(length);
}
